@@ -16,11 +16,11 @@ flatpak-builder \
 flatpak build ${builddir} meson \
                 --prefix=/app \
                 --libdir=/app/lib \
-                --buildtype=release \
+                --buildtype=debugoptimized \
                 -Dx11-backend=true \
                 -Dwayland-backend=true \
-                -Dprint-backends=file \
                 -Dbuild-tests=false \
+                -Dbuild-testsuite=false \
                 -Dbuild-examples=false \
                 -Dintrospection=disabled \
                 -Ddemos=true \
