@@ -91,7 +91,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
   }
 
   if (!gtk_widget_get_visible (window))
-    gtk_widget_show (window);
+    gtk_widget_set_visible (window, TRUE);
   else
     gtk_window_destroy (GTK_WINDOW (window));
 
