@@ -24,13 +24,12 @@
 #include "gdkglcontextprivate.h"
 #include "gdkdisplayprivate.h"
 #include "gdksurface.h"
-#include "gdkinternals.h"
 
 #include "gdkmacosdisplay.h"
 #include "gdkmacossurface.h"
 
 #import <OpenGL/OpenGL.h>
-#import <OpenGL/gl.h>
+#import <OpenGL/gl3.h>
 #import <AppKit/AppKit.h>
 
 G_BEGIN_DECLS
@@ -40,16 +39,14 @@ struct _GdkMacosGLContext
   GdkGLContext parent_instance;
 
   G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  NSOpenGLContext *gl_context;
+  CGLContextObj cgl_context;
   G_GNUC_END_IGNORE_DEPRECATIONS
 
-  NSWindow *dummy_window;
-  NSView *dummy_view;
+  GLuint texture;
+  GLuint target;
+  GLuint fbo;
 
-  cairo_region_t *damage;
-
-  guint is_attached : 1;
-  guint needs_resize : 1;
+  guint last_opaque : 1;
 };
 
 struct _GdkMacosGLContextClass
@@ -57,7 +54,6 @@ struct _GdkMacosGLContextClass
   GdkGLContextClass parent_class;
 };
 
-
 G_END_DECLS
 
 #endif /* __GDK_MACOS_GL_CONTEXT_PRIVATE_H__ */
