@@ -29,6 +29,7 @@
 #include "gdkdevice-win32.h"
 #include "gdkdevice-virtual.h"
 #include "gdkdevice-wintab.h"
+#include "gdkeventsprivate.h"
 #include "gdkinput-winpointer.h"
 #include "gdkdisplayprivate.h"
 #include "gdkdisplay-win32.h"
@@ -428,7 +429,7 @@ wintab_init_check (GdkDeviceManagerWin32 *device_manager)
   /* Create a dummy window to receive wintab events */
   wintab_window =
       _gdk_win32_display_create_surface (display,
-                                         GDK_SURFACE_TEMP,
+                                         GDK_SURFACE_DRAG,
                                          NULL,
                                          -100, -100, 2, 2);
 
