@@ -30,13 +30,15 @@
 
 #include "gdkeventsprivate.h"
 #include "gdkseatprivate.h"
-#include "gdk-private.h"
+#include "gdkprivate.h"
 
 #include <fcntl.h>
 #include <io.h>
 #include <math.h>
 #include <string.h>
 
+#define GDK_NOTE(a,b)
+
 /*
  * Support for OLE-2 drag and drop added at Archaeopteryx Software, 2001
  * For more information, do not contact Stephan R.A. Deibel (sdeibel@archaeopteryx.com),
@@ -212,7 +214,7 @@
 #include <shlobj.h>
 #include <shlguid.h>
 #include <objidl.h>
-#include "gdkintl.h"
+#include <glib/gi18n-lib.h>
 
 #include <gdk/gdk.h>
 #include <glib/gstdio.h>
@@ -1650,7 +1652,7 @@ create_drag_surface (GdkDisplay *display)
   GdkSurface *surface;
 
   surface = _gdk_win32_display_create_surface (display,
-                                               GDK_SURFACE_TEMP,
+                                               GDK_SURFACE_DRAG,
                                                NULL,
                                                0, 0, 100, 100);
 
