@@ -26,6 +26,7 @@
 #define __GDK_PRIVATE_WIN32_H__
 
 #include <gdk/gdkcursorprivate.h>
+#include <gdk/gdkdebugprivate.h>
 #include <gdk/win32/gdksurface-win32.h>
 #include <gdk/win32/gdkwin32display.h>
 #include <gdk/win32/gdkwin32screen.h>
@@ -37,6 +38,23 @@
 
 #include "config.h"
 
+
+/* Old debug macros */
+
+#ifdef G_ENABLE_DEBUG
+
+#define GDK_NOTE(type,action)                             \
+    G_STMT_START {                                        \
+      if (GDK_DEBUG_CHECK (type))                         \
+         { action; };                                     \
+    } G_STMT_END
+
+#else
+
+#define GDK_NOTE(type,action)
+
+#endif
+
 /* Make up for some minor w32api or MSVC6 header lossage */
 
 #ifndef PS_JOIN_MASK
@@ -312,9 +330,6 @@ void  _gdk_win32_end_modal_call (GdkWin32ModalOpKind kind);
  * Windows XP, thresholds alpha otherwise.
  */
 HICON _gdk_win32_texture_to_hicon  (GdkTexture *texture);
-HICON _gdk_win32_pixbuf_to_hcursor (GdkPixbuf *pixbuf,
-                                    int        x_hotspot,
-                                    int        y_hotspot);
 
 void _gdk_win32_display_init_cursors (GdkWin32Display     *display);
 void _gdk_win32_display_finalize_cursors (GdkWin32Display *display);
