@@ -27,7 +27,6 @@
 
 #include "gdksurfaceprivate.h"
 #include "gdkx11surface.h"
-#include "gdkinternals.h"
 
 #include <X11/Xlib.h>
 
@@ -87,7 +86,6 @@ struct _GdkX11Surface
   guint compute_size_source_id;
 
   cairo_surface_t *cairo_surface;
-  /* EGLSurface */ gpointer egl_surface;
   /* GLXDrawable */ XID glx_drawable;
   guint32 glx_frame_counter;
 
