@@ -24,6 +24,7 @@
 #include <gdk/gdkdisplayprivate.h>
 #include <gdk/gdkglcontextprivate.h>
 #include <gdk/gdksurfaceprivate.h>
+#include <glib/gi18n-lib.h>
 #include <gsk/gskdebugprivate.h>
 #include <gsk/gskrendererprivate.h>
 #include <gsk/gskrendernodeprivate.h>
@@ -93,6 +94,7 @@ gsk_gl_renderer_realize (GskRenderer  *renderer,
   GdkDisplay *display;
   gboolean ret = FALSE;
   gboolean debug_shaders = FALSE;
+  GdkGLAPI api;
 
   if (self->context != NULL)
     return TRUE;
@@ -115,6 +117,24 @@ gsk_gl_renderer_realize (GskRenderer  *renderer,
   if (!context || !gdk_gl_context_realize (context, error))
     goto failure;
 
+  api = gdk_gl_context_get_api (context);
+  if (api == GDK_GL_API_GLES)
+    {
+      gdk_gl_context_make_current (context);
+
+      if (!gdk_gl_context_has_vertex_half_float (context))
+        {
+          int major, minor;
+
+          gdk_gl_context_get_version (context, &major, &minor);
+          g_set_error (error,
+                       GDK_GL_ERROR, GDK_GL_ERROR_NOT_AVAILABLE,
+                       _("This GLES %d.%d implementation does not support half-float vertex data"),
+                       major, minor);
+          goto failure;
+        }
+    }
+
 #ifdef G_ENABLE_DEBUG
   if (GSK_RENDERER_DEBUG_CHECK (GSK_RENDERER (self), SHADERS))
     debug_shaders = TRUE;
@@ -138,6 +158,10 @@ failure:
 
   gdk_profiler_end_mark (start_time, "realize GskGLRenderer", NULL);
 
+  /* Assert either all or no state was set */
+  g_assert ((ret && self->driver != NULL && self->context != NULL && self->command_queue != NULL) ||
+            (!ret && self->driver == NULL && self->context == NULL && self->command_queue == NULL));
+
   return ret;
 }
 
@@ -188,6 +212,65 @@ get_render_region (GdkSurface   *surface,
   return cairo_region_create_rectangle (&extents);
 }
 
+static gboolean
+update_area_requires_clear (GdkSurface           *surface,
+                            const cairo_region_t *update_area)
+{
+  cairo_rectangle_int_t rect;
+  guint n_rects;
+
+  g_assert (GDK_IS_SURFACE (surface));
+
+  /* No opaque region, assume we have to clear */
+  if (surface->opaque_region == NULL)
+    return TRUE;
+
+  /* If the update_area is the whole surface, then clear it
+   * because many drivers optimize for this by avoiding extra
+   * work to reload any contents.
+   */
+  if (update_area == NULL)
+    return TRUE;
+
+  if (cairo_region_num_rectangles (update_area) == 1)
+    {
+      cairo_region_get_rectangle (update_area, 0, &rect);
+
+      if (rect.x == 0 &&
+          rect.y == 0 &&
+          rect.width == surface->width &&
+          rect.height == surface->height)
+        return TRUE;
+    }
+
+  /* If the entire surface is opaque, then we can skip clearing
+   * (with the exception of full surface clearing above).
+   */
+  if (cairo_region_num_rectangles (surface->opaque_region) == 1)
+    {
+      cairo_region_get_rectangle (surface->opaque_region, 0, &rect);
+
+      if (rect.x == 0 &&
+          rect.y == 0 &&
+          rect.width == surface->width &&
+          rect.height == surface->height)
+        return FALSE;
+    }
+
+  /* If any update_area rectangle overlaps our transparent
+   * regions, then we need to clear the area.
+   */
+  n_rects = cairo_region_num_rectangles (update_area);
+  for (guint i = 0; i < n_rects; i++)
+    {
+      cairo_region_get_rectangle (update_area, i, &rect);
+      if (cairo_region_contains_rectangle (surface->opaque_region, &rect) != CAIRO_REGION_OVERLAP_IN)
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 gsk_gl_renderer_render (GskRenderer          *renderer,
                         GskRenderNode        *root,
@@ -198,6 +281,7 @@ gsk_gl_renderer_render (GskRenderer          *renderer,
   graphene_rect_t viewport;
   GskGLRenderJob *job;
   GdkSurface *surface;
+  gboolean clear_framebuffer;
   float scale_factor;
 
   g_assert (GSK_IS_GL_RENDERER (renderer));
@@ -219,9 +303,10 @@ gsk_gl_renderer_render (GskRenderer          *renderer,
 
   /* Must be called *AFTER* gdk_draw_context_begin_frame() */
   render_region = get_render_region (surface, self->context);
+  clear_framebuffer = update_area_requires_clear (surface, render_region);
 
   gsk_gl_driver_begin_frame (self->driver, self->command_queue);
-  job = gsk_gl_render_job_new (self->driver, &viewport, scale_factor, render_region, 0);
+  job = gsk_gl_render_job_new (self->driver, &viewport, scale_factor, render_region, 0, clear_framebuffer);
 #ifdef G_ENABLE_DEBUG
   if (GSK_RENDERER_DEBUG_CHECK (GSK_RENDERER (self), FALLBACK))
     gsk_gl_render_job_set_debug_fallback (job, TRUE);
@@ -245,10 +330,9 @@ gsk_gl_renderer_render_texture (GskRenderer           *renderer,
   GskGLRenderer *self = (GskGLRenderer *)renderer;
   GskGLRenderTarget *render_target;
   GskGLRenderJob *job;
-  GdkTexture *texture = NULL;
+  GdkTexture *texture;
   guint texture_id;
-  int width;
-  int height;
+  int width, height, max_size;
   int format;
 
   g_assert (GSK_IS_GL_RENDERER (renderer));
@@ -256,8 +340,43 @@ gsk_gl_renderer_render_texture (GskRenderer           *renderer,
 
   width = ceilf (viewport->size.width);
   height = ceilf (viewport->size.height);
+  max_size = self->command_queue->max_texture_size;
+  if (width > max_size || height > max_size)
+    {
+      gsize x, y, size, stride;
+      GBytes *bytes;
+      guchar *data;
 
-  format = gsk_render_node_prefers_high_depth (root) ? GL_RGBA32F : GL_RGBA8;
+      stride = width * 4;
+      size = stride * height;
+      data = g_malloc_n (stride, height);
+
+      for (y = 0; y < height; y += max_size)
+        {
+          for (x = 0; x < width; x += max_size)
+            {
+              texture = gsk_gl_renderer_render_texture (renderer, root,
+                                                        &GRAPHENE_RECT_INIT (x, y,
+                                                                             MIN (max_size, viewport->size.width - x),
+                                                                             MIN (max_size, viewport->size.height - y)));
+              gdk_texture_download (texture,
+                                    data + stride * y + x * 4,
+                                    stride);
+              g_object_unref (texture);
+            }
+        }
+
+      bytes = g_bytes_new_take (data, size);
+      texture = gdk_memory_texture_new (width, height, GDK_MEMORY_DEFAULT, bytes, stride);
+      g_bytes_unref (bytes);
+      return texture;
+    }
+
+  if (gsk_render_node_prefers_high_depth (root) &&
+      gdk_gl_context_check_version (self->context, 3, 0, 3, 0))
+    format = GL_RGBA32F;
+  else 
+    format = GL_RGBA8;
 
   gdk_gl_context_make_current (self->context);
 
@@ -268,7 +387,7 @@ gsk_gl_renderer_render_texture (GskRenderer           *renderer,
                                           &render_target))
     {
       gsk_gl_driver_begin_frame (self->driver, self->command_queue);
-      job = gsk_gl_render_job_new (self->driver, viewport, 1, NULL, render_target->framebuffer_id);
+      job = gsk_gl_render_job_new (self->driver, viewport, 1, NULL, render_target->framebuffer_id, TRUE);
 #ifdef G_ENABLE_DEBUG
       if (GSK_RENDERER_DEBUG_CHECK (GSK_RENDERER (self), FALLBACK))
         gsk_gl_render_job_set_debug_fallback (job, TRUE);
@@ -281,6 +400,10 @@ gsk_gl_renderer_render_texture (GskRenderer           *renderer,
 
       gsk_gl_driver_after_frame (self->driver);
     }
+  else
+    {
+      g_assert_not_reached ();
+    }
 
   return g_steal_pointer (&texture);
 }
@@ -288,11 +411,11 @@ gsk_gl_renderer_render_texture (GskRenderer           *renderer,
 static void
 gsk_gl_renderer_dispose (GObject *object)
 {
-#ifdef G_ENABLE_DEBUG
   GskGLRenderer *self = (GskGLRenderer *)object;
 
-  g_assert (self->driver == NULL);
-#endif
+  if (self->driver != NULL)
+    g_critical ("Attempt to dispose %s without calling gsk_renderer_unrealize()",
+                G_OBJECT_TYPE_NAME (self));
 
   G_OBJECT_CLASS (gsk_gl_renderer_parent_class)->dispose (object);
 }
