@@ -13,7 +13,7 @@ typedef struct _GskRenderNodeClass GskRenderNodeClass;
  * We don't add an "n-types" value to avoid having to handle
  * it in every single switch.
  */
-#define GSK_RENDER_NODE_TYPE_N_TYPES    (GSK_GL_SHADER_NODE + 1)
+#define GSK_RENDER_NODE_TYPE_N_TYPES    (GSK_MASK_NODE + 1)
 
 extern GType gsk_render_node_types[];
 
@@ -27,6 +27,9 @@ struct _GskRenderNode
   gatomicrefcount ref_count;
 
   graphene_rect_t bounds;
+
+  guint prefers_high_depth : 1;
+  guint offscreen_for_opacity : 1;
 };
 
 struct _GskRenderNodeClass
@@ -109,6 +112,11 @@ GskRenderNode ** gsk_container_node_get_children        (const GskRenderNode *no
 void             gsk_transform_node_get_translate       (const GskRenderNode *node,
                                                          float               *dx,
                                                          float               *dy);
+gboolean       gsk_render_node_prefers_high_depth       (const GskRenderNode *node);
+
+gboolean       gsk_container_node_is_disjoint           (const GskRenderNode *node);
+
+gboolean       gsk_render_node_use_offscreen_for_opacity (const GskRenderNode *node);
 
 
 G_END_DECLS
