@@ -45,6 +45,11 @@
  * [vfunc@Gtk.Accessible.get_next_accessible_sibling] virtual functions.
  * Note that you can not create a top-level accessible object as of now,
  * which means that you must always have a parent accessible object.
+ * Also note that when an accessible object does not correspond to a widget,
+ * and it has children, whose implementation you don't control,
+ * it is necessary to ensure the correct shape of the a11y tree
+ * by calling [method@Gtk.Accessible.set_accessible_parent] and
+ * updating the sibling by [method@Gtk.Accessible.update_next_accessible_sibling].
  */
 
 #include "config.h"
@@ -86,9 +91,11 @@ gtk_accessible_default_init (GtkAccessibleInterface *iface)
  * gtk_accessible_get_at_context:
  * @self: a `GtkAccessible`
  *
- * Retrieves the `GtkATContext` for the given `GtkAccessible`.
+ * Retrieves the accessible implementation for the given `GtkAccessible`.
  *
- * Returns: (transfer none): the `GtkATContext`
+ * Returns: (transfer full): the accessible implementation object
+ *
+ * Since: 4.10
  */
 GtkATContext *
 gtk_accessible_get_at_context (GtkAccessible *self)
@@ -102,11 +109,11 @@ gtk_accessible_get_at_context (GtkAccessible *self)
  * gtk_accessible_get_accessible_parent:
  * @self: a `GtkAccessible`
  *
- * Retrieves the accessible accessible for an accessible object
+ * Retrieves the accessible parent for an accessible object.
  *
- * This function returns `NULL` for top level widgets
+ * This function returns `NULL` for top level widgets.
  *
- * Returns: (transfer none) (nullable): the accessible parent
+ * Returns: (transfer full) (nullable): the accessible parent
  *
  * Since: 4.10
  */
@@ -115,7 +122,97 @@ gtk_accessible_get_accessible_parent (GtkAccessible *self)
 {
   g_return_val_if_fail (GTK_IS_ACCESSIBLE (self), NULL);
 
-  return GTK_ACCESSIBLE_GET_IFACE (self)->get_accessible_parent (self);
+  GtkATContext *context;
+  GtkAccessible *parent = NULL;
+
+  context = gtk_accessible_get_at_context (self);
+  if (context != NULL)
+    {
+      parent = gtk_at_context_get_accessible_parent (context);
+      g_object_unref (context);
+    }
+
+  if (parent != NULL)
+    return g_object_ref (parent);
+  else
+    return GTK_ACCESSIBLE_GET_IFACE (self)->get_accessible_parent (self);
+}
+
+/**
+ * gtk_accessible_set_accessible_parent:
+ * @self: an accessible object
+ * @parent: (nullable): the parent accessible object
+ * @next_sibling: (nullable): the sibling accessible object
+ *
+ * Sets the parent and sibling of an accessible object.
+ *
+ * This function is meant to be used by accessible implementations that are
+ * not part of the widget hierarchy, and but act as a logical bridge between
+ * widgets. For instance, if a widget creates an object that holds metadata
+ * for each child, and you want that object to implement the `GtkAccessible`
+ * interface, you will use this function to ensure that the parent of each
+ * child widget is the metadata object, and the parent of each metadata
+ * object is the container widget.
+ *
+ * Since: 4.10
+ */
+void
+gtk_accessible_set_accessible_parent (GtkAccessible *self,
+                                      GtkAccessible *parent,
+                                      GtkAccessible *next_sibling)
+{
+  g_return_if_fail (GTK_IS_ACCESSIBLE (self));
+  g_return_if_fail (parent == NULL || GTK_IS_ACCESSIBLE (parent));
+  g_return_if_fail (next_sibling == NULL || GTK_IS_ACCESSIBLE (parent));
+
+  GtkATContext *context;
+
+  context = gtk_accessible_get_at_context (self);
+  if (context != NULL)
+    {
+      gtk_at_context_set_accessible_parent (context, parent);
+      gtk_at_context_set_next_accessible_sibling (context, next_sibling);
+      g_object_unref (context);
+    }
+}
+
+/**
+ * gtk_accessible_update_next_accessible_sibling:
+ * @self: a `GtkAccessible`
+ * @new_sibling: (nullable): the new next accessible sibling to set
+ *
+ * Updates the next accessible sibling of @self.
+ *
+ * That might be useful when a new child of a custom `GtkAccessible`
+ * is created, and it needs to be linked to a previous child.
+ *
+ * Since: 4.10
+ */
+void
+gtk_accessible_update_next_accessible_sibling (GtkAccessible *self,
+                                               GtkAccessible *new_sibling)
+{
+  GtkATContext *context;
+  GtkAccessible *parent;
+
+  g_return_if_fail (GTK_IS_ACCESSIBLE (self));
+
+  context = gtk_accessible_get_at_context (self);
+  if (context == NULL)
+    return;
+
+  parent = gtk_at_context_get_accessible_parent (context);
+  if (parent == NULL)
+    {
+      g_object_unref (context);
+      g_critical ("Failed to update next accessible sibling: no parent accessible set for this accessible");
+      return;
+    }
+
+  gtk_at_context_set_next_accessible_sibling (context, new_sibling);
+
+  g_object_unref (parent);
+  g_object_unref (context);
 }
 
 /**
@@ -124,7 +221,7 @@ gtk_accessible_get_accessible_parent (GtkAccessible *self)
  *
  * Retrieves the first accessible child of an accessible object.
  *
- * Returns: (transfer none) (nullable): the first accessible child
+ * Returns: (transfer full) (nullable): the first accessible child
  *
  * since: 4.10
  */
@@ -142,7 +239,7 @@ gtk_accessible_get_first_accessible_child (GtkAccessible *self)
  *
  * Retrieves the next accessible sibling of an accessible object
  *
- * Returns: (transfer none) (nullable): the next accessible sibling
+ * Returns: (transfer full) (nullable): the next accessible sibling
  *
  * since: 4.10
  */
@@ -151,7 +248,22 @@ gtk_accessible_get_next_accessible_sibling (GtkAccessible *self)
 {
   g_return_val_if_fail (GTK_IS_ACCESSIBLE (self), NULL);
 
-  return GTK_ACCESSIBLE_GET_IFACE (self)->get_next_accessible_sibling (self);
+  GtkATContext *context;
+  GtkAccessible *sibling = NULL;
+
+  context = gtk_accessible_get_at_context (self);
+  if (context != NULL && gtk_at_context_get_accessible_parent (context) != NULL)
+    {
+      sibling = gtk_at_context_get_next_accessible_sibling (context);
+      if (sibling != NULL)
+        sibling = g_object_ref (sibling);
+    }
+  else
+    sibling = GTK_ACCESSIBLE_GET_IFACE (self)->get_next_accessible_sibling (self);
+
+  g_clear_object (&context);
+
+  return sibling;
 }
 
 /**
@@ -165,13 +277,21 @@ gtk_accessible_get_next_accessible_sibling (GtkAccessible *self)
 GtkAccessibleRole
 gtk_accessible_get_accessible_role (GtkAccessible *self)
 {
-  GtkAccessibleRole role;
+  GtkAccessibleRole role = GTK_ACCESSIBLE_ROLE_NONE;
 
   g_return_val_if_fail (GTK_IS_ACCESSIBLE (self), GTK_ACCESSIBLE_ROLE_NONE);
 
   GtkATContext *context = gtk_accessible_get_at_context (self);
-  if (context != NULL && gtk_at_context_is_realized (context))
-    return gtk_at_context_get_accessible_role (context);
+  if (context != NULL)
+    {
+      if (gtk_at_context_is_realized (context))
+        role = gtk_at_context_get_accessible_role (context);
+
+      g_object_unref (context);
+
+      if (role != GTK_ACCESSIBLE_ROLE_NONE)
+        return role;
+    }
 
   g_object_get (G_OBJECT (self), "accessible-role", &role, NULL);
 
@@ -245,6 +365,8 @@ gtk_accessible_update_state (GtkAccessible      *self,
 
 out:
   va_end (args);
+
+  g_object_unref (context);
 }
 
 /**
@@ -298,6 +420,7 @@ gtk_accessible_update_state_value (GtkAccessible      *self,
     }
 
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -321,6 +444,7 @@ gtk_accessible_reset_state (GtkAccessible      *self,
 
   gtk_at_context_set_accessible_state (context, state, NULL);
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -392,6 +516,8 @@ gtk_accessible_update_property (GtkAccessible         *self,
 
 out:
   va_end (args);
+
+  g_object_unref (context);
 }
 
 /**
@@ -445,6 +571,7 @@ gtk_accessible_update_property_value (GtkAccessible         *self,
     }
 
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -468,6 +595,7 @@ gtk_accessible_reset_property (GtkAccessible         *self,
 
   gtk_at_context_set_accessible_property (context, property, NULL);
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -539,6 +667,8 @@ gtk_accessible_update_relation (GtkAccessible         *self,
 
 out:
   va_end (args);
+
+  g_object_unref (context);
 }
 
 /**
@@ -567,6 +697,8 @@ gtk_accessible_update_relation_value (GtkAccessible         *self,
   g_return_if_fail (n_relations > 0);
 
   context = gtk_accessible_get_at_context (self);
+  if (context == NULL)
+    return;
 
   for (int i = 0; i < n_relations; i++)
     {
@@ -585,15 +717,14 @@ gtk_accessible_update_relation_value (GtkAccessible         *self,
           break;
         }
 
-      if (context)
-        gtk_at_context_set_accessible_relation (context, relation, real_value);
+      gtk_at_context_set_accessible_relation (context, relation, real_value);
 
       if (real_value != NULL)
         gtk_accessible_value_unref (real_value);
     }
 
-  if (context)
-    gtk_at_context_update (context);
+  gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -617,6 +748,7 @@ gtk_accessible_reset_relation (GtkAccessible         *self,
 
   gtk_at_context_set_accessible_relation (context, relation, NULL);
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 static const char *role_names[] = {
@@ -698,6 +830,7 @@ static const char *role_names[] = {
   [GTK_ACCESSIBLE_ROLE_TREE_ITEM] = NC_("accessibility", "tree item"),
   [GTK_ACCESSIBLE_ROLE_WIDGET] = NC_("accessibility", "widget"),
   [GTK_ACCESSIBLE_ROLE_WINDOW] = NC_("accessibility", "window"),
+  [GTK_ACCESSIBLE_ROLE_TOGGLE_BUTTON] = NC_("accessibility", "toggle button"),
 };
 
 /*< private >
@@ -777,13 +910,22 @@ gtk_accessible_platform_changed (GtkAccessible               *self,
 
   /* propagate changes up from ignored widgets */
   if (gtk_accessible_get_accessible_role (self) == GTK_ACCESSIBLE_ROLE_NONE)
-    context = gtk_accessible_get_at_context (gtk_accessible_get_accessible_parent (self));
+    {
+      GtkAccessible *parent = gtk_accessible_get_accessible_parent (self);
+
+      if (parent != NULL)
+        {
+          context = gtk_accessible_get_at_context (parent);
+          g_object_unref (parent);
+        }
+    }
 
   if (context == NULL)
     return;
 
   gtk_at_context_platform_changed (context, change);
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -837,6 +979,7 @@ gtk_accessible_bounds_changed (GtkAccessible *self)
     return;
 
   gtk_at_context_bounds_changed (context);
+  g_object_unref (context);
 }
 
 /**
@@ -889,6 +1032,7 @@ gtk_accessible_should_present (GtkAccessible *self)
 {
   GtkAccessibleRole role;
   GtkATContext *context;
+  gboolean res = TRUE;
 
   if (GTK_IS_WIDGET (self) &&
       !gtk_widget_get_visible (GTK_WIDGET (self)))
@@ -909,10 +1053,12 @@ gtk_accessible_should_present (GtkAccessible *self)
 
       value = gtk_at_context_get_accessible_state (context, GTK_ACCESSIBLE_STATE_HIDDEN);
       if (gtk_boolean_accessible_value_get (value))
-        return FALSE;
+        res = FALSE;
     }
 
-  return TRUE;
+  g_object_unref (context);
+
+  return res;
 }
 
 void
@@ -926,15 +1072,24 @@ gtk_accessible_update_children (GtkAccessible           *self,
       gtk_widget_get_root (GTK_WIDGET (self)) == NULL)
     return;
 
-  context = gtk_accessible_get_at_context (self);
-
   /* propagate changes up from ignored widgets */
   if (gtk_accessible_get_accessible_role (self) == GTK_ACCESSIBLE_ROLE_NONE)
-    context = gtk_accessible_get_at_context (gtk_accessible_get_accessible_parent (self));
+    {
+      GtkAccessible *parent = gtk_accessible_get_accessible_parent (self);
+
+      context = gtk_accessible_get_at_context (parent);
+
+      g_object_unref (parent);
+    }
+  else
+    {
+      context = gtk_accessible_get_at_context (self);
+    }
 
   if (context == NULL)
     return;
 
   gtk_at_context_child_changed (context, 1 << state, child);
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
