@@ -50,7 +50,6 @@
 
 #include "gtkwidgetprivate.h"
 #include "gtkprivate.h"
-#include "gtkintl.h"
 
 
 typedef struct _GtkAspectFrameClass GtkAspectFrameClass;
@@ -455,7 +454,7 @@ gtk_aspect_frame_get_ratio (GtkAspectFrame *self)
  * gtk_aspect_frame_set_obey_child: (attributes org.gtk.Method.set_propery=obey-child)
  * @self: a `GtkAspectFrame`
  * @obey_child: If %TRUE, @ratio is ignored, and the aspect
- *    ratio is taken from the requistion of the child.
+ *    ratio is taken from the requisition of the child.
  *
  * Sets whether the aspect ratio of the child's size
  * request should override the set aspect ratio of
@@ -663,7 +662,7 @@ gtk_aspect_frame_set_child (GtkAspectFrame  *self,
  *
  * Gets the child widget of @self.
  *
- * Returns: (nullable) (transfer none): the child widget of self@
+ * Returns: (nullable) (transfer none): the child widget of @self
  */
 GtkWidget *
 gtk_aspect_frame_get_child (GtkAspectFrame *self)
