@@ -29,7 +29,6 @@ G_BEGIN_DECLS
 
 #define GTK_TYPE_BUILDABLE            (gtk_buildable_get_type ())
 #define GTK_BUILDABLE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_BUILDABLE, GtkBuildable))
-#define GTK_BUILDABLE_CLASS(obj)      (G_TYPE_CHECK_CLASS_CAST ((obj), GTK_TYPE_BUILDABLE, GtkBuildableIface))
 #define GTK_IS_BUILDABLE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_BUILDABLE))
 #define GTK_BUILDABLE_GET_IFACE(obj)  (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GTK_TYPE_BUILDABLE, GtkBuildableIface))
 
@@ -39,6 +38,12 @@ typedef struct _GtkBuildableIface GtkBuildableIface;
 typedef struct _GtkBuildableParseContext      GtkBuildableParseContext;
 typedef struct _GtkBuildableParser GtkBuildableParser;
 
+/**
+ * GtkBuildableParseContext:
+ *
+ * An opaque context struct for `GtkBuildableParser`.
+ */
+
 /**
  * GtkBuildableParser:
  * @start_element: function called for open elements
@@ -46,7 +51,7 @@ typedef struct _GtkBuildableParser GtkBuildableParser;
  * @text: function called for character data
  * @error: function called on error
  *
- * A sub-parser for #GtkBuildable implementations.
+ * A sub-parser for `GtkBuildable` implementations.
  */
 struct _GtkBuildableParser
 {
@@ -86,32 +91,32 @@ struct _GtkBuildableParser
 /**
  * GtkBuildableIface:
  * @g_iface: the parent class
- * @set_id: Stores the id attribute given in the GtkBuilder UI definition.
- *  #GtkWidget stores the name as object data. Implement this method if your
- *  object has some notion of “ID” and it makes sense to map the XML id
- *  attribute to it.
+ * @set_id: Stores the id attribute given in the `GtkBuilder` UI definition.
+ *   `GtkWidget` stores the name as object data. Implement this method if your
+ *   object has some notion of “ID” and it makes sense to map the XML id
+ *   attribute to it.
  * @get_id: The getter corresponding to @set_id. Implement this
- *  if you implement @set_id.
+ *   if you implement @set_id.
  * @add_child: Adds a child. The @type parameter can be used to
- *  differentiate the kind of child. #GtkWidget implements this
- *  to add event controllers to the widget, #GtkNotebook uses
- *  the @type to distinguish between page labels (of type "page-label")
- *  and normal children.
+ *   differentiate the kind of child. `GtkWidget` implements this
+ *   to add event controllers to the widget, `GtkNotebook` uses
+ *   the @type to distinguish between page labels (of type "page-label")
+ *   and normal children.
  * @set_buildable_property: Sets a property of a buildable object.
  *  It is normally not necessary to implement this, g_object_set_property()
- *  is used by default. #GtkWindow implements this to delay showing itself
- *  (i.e. setting the #GtkWidget:visible property) until the whole interface
- *  is created.
+ *  is used by default. `GtkWindow` implements this to delay showing itself
+ *  (i.e. setting the [property@Gtk.Widget:visible] property) until the whole
+ *  interface is created.
  * @construct_child: Constructs a child of a buildable that has been
  *  specified as “constructor” in the UI definition. This can be used to
- *  reference a widget created in a <ui> tag which is outside
+ *  reference a widget created in a `<ui>` tag which is outside
  *  of the normal GtkBuilder UI definition hierarchy.  A reference to the
  *  constructed object is returned and becomes owned by the caller.
  * @custom_tag_start: Implement this if the buildable needs to parse
- *  content below <child>. To handle an element, the implementation
+ *  content below `<child>`. To handle an element, the implementation
  *  must fill in the @parser and @user_data and return %TRUE.
- *  #GtkWidget implements this to parse accessible attributes specified
- *  in <accessibility> elements.
+ *  `GtkWidget` implements this to parse accessible attributes specified
+ *  in `<accessibility>` elements.
  *  Note that @user_data must be freed in @custom_tag_end or @custom_finished.
  * @custom_tag_end: Called for the end tag of each custom element that is
  *  handled by the buildable (see @custom_tag_start).
@@ -119,17 +124,17 @@ struct _GtkBuildableParser
  *  when the builder finishes parsing (see @custom_tag_start)
  * @parser_finished: Called when a builder finishes the parsing
  *  of a UI definition. It is normally not necessary to implement this,
- *  unless you need to perform special cleanup actions. #GtkWindow sets
- *  the #GtkWidget:visible property here.
+ *  unless you need to perform special cleanup actions. `GtkWindow` sets
+ *  the `GtkWidget:visible` property here.
  * @get_internal_child: Returns an internal child of a buildable.
- *  #GtkDialog implements this to give access to its @vbox, making
+ *  `GtkDialog` implements this to give access to its @vbox, making
  *  it possible to add children to the vbox in a UI definition.
  *  Implement this if the buildable has internal children that may
  *  need to be accessed from a UI definition.
  *
- * The #GtkBuildableIface interface contains method that are
- * necessary to allow #GtkBuilder to construct an object from
- * a #GtkBuilder UI definition.
+ * The `GtkBuildableIface` interface contains methods that are
+ * necessary to allow `GtkBuilder` to construct an object from
+ * a `GtkBuilder` UI definition.
  */
 struct _GtkBuildableIface
 {
@@ -142,8 +147,8 @@ struct _GtkBuildableIface
 
   /**
    * GtkBuildableIface::add_child:
-   * @buildable: a #GtkBuildable
-   * @builder: a #GtkBuilder
+   * @buildable: a `GtkBuildable`
+   * @builder: a `GtkBuilder`
    * @child: child to add
    * @type: (nullable): kind of child or %NULL
    *
@@ -164,11 +169,11 @@ struct _GtkBuildableIface
 
   /**
    * GtkBuildableIface::custom_tag_start:
-   * @buildable: a #GtkBuildable
-   * @builder: a #GtkBuilder used to construct this object
+   * @buildable: a `GtkBuildable`
+   * @builder: a `GtkBuilder` used to construct this object
    * @child: (nullable): child object or %NULL for non-child tags
    * @tagname: name of tag
-   * @parser: (out): a #GtkBuildableParser to fill in
+   * @parser: (out): a `GtkBuildableParser` to fill in
    * @data: (out): return location for user data that will be passed in
    *   to parser functions
    *
@@ -185,8 +190,8 @@ struct _GtkBuildableIface
                                             gpointer           *data);
   /**
    * GtkBuildableIface::custom_tag_end:
-   * @buildable: A #GtkBuildable
-   * @builder: #GtkBuilder used to construct this object
+   * @buildable: A `GtkBuildable`
+   * @builder: `GtkBuilder` used to construct this object
    * @child: (nullable): child object or %NULL for non-child tags
    * @tagname: name of tag
    * @data: user data that will be passed in to parser functions
@@ -201,8 +206,8 @@ struct _GtkBuildableIface
                                             gpointer            data);
    /**
     * GtkBuildableIface::custom_finished:
-    * @buildable: a #GtkBuildable
-    * @builder: a #GtkBuilder
+    * @buildable: a `GtkBuildable`
+    * @builder: a `GtkBuilder`
     * @child: (nullable): child object or %NULL for non-child tags
     * @tagname: the name of the tag
     * @data: user data created in custom_tag_start
@@ -220,8 +225,8 @@ struct _GtkBuildableIface
 
   /**
    * GtkBuildableIface::get_internal_child:
-   * @buildable: a #GtkBuildable
-   * @builder: a #GtkBuilder
+   * @buildable: a `GtkBuildable`
+   * @builder: a `GtkBuilder`
    * @childname: name of child
    *
    * Retrieves the internal child called @childname of the @buildable object.
@@ -255,6 +260,8 @@ void          gtk_buildable_parse_context_get_position      (GtkBuildableParseCo
                                                              int                      *line_number,
                                                              int                      *char_number);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GtkBuildable, g_object_unref)
+
 G_END_DECLS
 
 #endif /* __GTK_BUILDABLE_H__ */
