@@ -19,7 +19,9 @@
 
 #include "config.h"
 
-#include "gtkcssparserprivate.h"
+#include <gtk/css/gtkcss.h>
+#include "gtk/css/gtkcsstokenizerprivate.h"
+#include "gtk/css/gtkcssparserprivate.h"
 #include "gtkcssnumbervalueprivate.h"
 #include "gtkcssfontvariationsvalueprivate.h"
 
