@@ -19,12 +19,12 @@
 
 #include "gtkcsskeyframesprivate.h"
 
+#include "gtkcssstyleprivate.h"
 #include "gtkcssarrayvalueprivate.h"
 #include "gtkcssshorthandpropertyprivate.h"
 #include "gtkcssstylepropertyprivate.h"
 #include "gtkstylepropertyprivate.h"
 
-/* XXX: For g_memdup2() */
 #include "gtkprivate.h"
 
 #include <stdlib.h>
@@ -97,7 +97,8 @@ gtk_css_keyframes_add_keyframe (GtkCssKeyframes *keyframes,
               KEYFRAMES_VALUE (keyframes, k, p) = NULL;
 
               /* XXX: GC properties that are now unset
-               * in all keyframes? */
+               * in all keyframes?
+               */
             }
           return k;
         }
