@@ -28,6 +28,7 @@
 static GtkCssValue *        gtk_css_calc_value_new         (guint n_terms);
 static GtkCssValue *        gtk_css_calc_value_new_sum     (GtkCssValue *a,
                                                             GtkCssValue *b);
+static gsize                gtk_css_value_calc_get_size    (gsize n_terms);
 
 enum {
   TYPE_CALC = 0,
@@ -69,11 +70,22 @@ gtk_css_calc_value_new_from_array (GtkCssValue **values,
   return result;
 }
 
-
 static void
-gtk_css_value_number_free (GtkCssValue *value)
+gtk_css_value_number_free (GtkCssValue *number)
 {
-  g_slice_free (GtkCssValue, value);
+  if (number->type == TYPE_CALC)
+    {
+      const guint n_terms = number->calc.n_terms;
+
+      for (guint i = 0; i < n_terms; i++)
+        _gtk_css_value_unref (number->calc.terms[i]);
+
+      g_slice_free1 (gtk_css_value_calc_get_size (n_terms), number);
+    }
+  else
+    {
+      g_slice_free (GtkCssValue, number);
+    }
 }
 
 static double
@@ -368,6 +380,9 @@ gtk_css_dimension_value_new (double     value,
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 2 }} },
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 3 }} },
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 4 }} },
+    { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 5 }} },
+    { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 6 }} },
+    { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 7 }} },
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 8 }} },
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 16 }} }, /* Icon size default */
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 32 }} },
@@ -406,16 +421,18 @@ gtk_css_dimension_value_new (double     value,
           value == 1 ||
           value == 2 ||
           value == 3 ||
-          value == 4)
+          value == 4 ||
+          value == 5 ||
+          value == 6 ||
+          value == 7 ||
+          value == 8)
         return _gtk_css_value_ref (&px_singletons[(int) value]);
-      if (value == 8)
-        return _gtk_css_value_ref (&px_singletons[5]);
       if (value == 16)
-        return _gtk_css_value_ref (&px_singletons[6]);
+        return _gtk_css_value_ref (&px_singletons[9]);
       if (value == 32)
-        return _gtk_css_value_ref (&px_singletons[7]);
+        return _gtk_css_value_ref (&px_singletons[10]);
       if (value == 64)
-        return _gtk_css_value_ref (&px_singletons[8]);
+        return _gtk_css_value_ref (&px_singletons[11]);
 
       break;
 
@@ -474,7 +491,7 @@ gtk_css_dimension_value_new (double     value,
  * match.
  *
  * Returns: Magic value determining placement when printing calc()
- *     expression.
+ *   expression.
  */
 static int
 gtk_css_number_value_get_calc_term_order (const GtkCssValue *value)
@@ -704,7 +721,8 @@ gtk_css_number_value_can_parse (GtkCssParser *parser)
       || gtk_css_parser_has_token (parser, GTK_CSS_TOKEN_PERCENTAGE)
       || gtk_css_parser_has_token (parser, GTK_CSS_TOKEN_SIGNED_INTEGER_DIMENSION)
       || gtk_css_parser_has_token (parser, GTK_CSS_TOKEN_SIGNLESS_INTEGER_DIMENSION)
-      || gtk_css_parser_has_token (parser, GTK_CSS_TOKEN_DIMENSION)
+      || gtk_css_parser_has_token (parser, GTK_CSS_TOKEN_SIGNED_DIMENSION)
+      || gtk_css_parser_has_token (parser, GTK_CSS_TOKEN_SIGNLESS_DIMENSION)
       || gtk_css_parser_has_function (parser, "calc");
 }
 
@@ -745,11 +763,8 @@ _gtk_css_number_value_get (const GtkCssValue *value,
 gboolean
 gtk_css_dimension_value_is_zero (const GtkCssValue *value)
 {
-  if (!value)
-    return TRUE;
-
-  if (value->class != &GTK_CSS_VALUE_NUMBER)
-    return FALSE;
+  g_assert (value != 0);
+  g_assert (value->class == &GTK_CSS_VALUE_NUMBER);
 
   if (value->type != TYPE_DIMENSION)
     return FALSE;
