@@ -142,7 +142,7 @@ gtk_css_transform_init_identity (GtkCssTransform     *transform,
     case GTK_CSS_TRANSFORM_ROTATE:
       transform->rotate.x = _gtk_css_number_value_new (0, GTK_CSS_NUMBER);
       transform->rotate.y = _gtk_css_number_value_new (0, GTK_CSS_NUMBER);
-      transform->rotate.z = _gtk_css_number_value_new (0, GTK_CSS_NUMBER);
+      transform->rotate.z = _gtk_css_number_value_new (1, GTK_CSS_NUMBER);
       transform->rotate.angle = _gtk_css_number_value_new (0, GTK_CSS_DEG);
       break;
     case GTK_CSS_TRANSFORM_SCALE:
@@ -287,6 +287,7 @@ gtk_css_transform_compute (GtkCssTransform  *dest,
   switch (src->type)
     {
     case GTK_CSS_TRANSFORM_MATRIX:
+      memcpy (dest, src, sizeof (GtkCssTransform));
       return TRUE;
     case GTK_CSS_TRANSFORM_TRANSLATE:
       dest->translate.x = _gtk_css_value_compute (src->translate.x, property_id, provider, style, parent_style);
@@ -942,7 +943,7 @@ _gtk_css_transform_value_parse (GtkCssParser *parser)
 
   while (TRUE)
     {
-      GtkCssTransform transform;
+      GtkCssTransform transform = { 0, };
 
       if (gtk_css_parser_has_function (parser, "matrix"))
         {
